-- add options window for configuration

-- ?? use fontMetric to automatically size display and center - automatically add multiple lines for wrapping when width:height ratio exceeds the displayWidth:displayHeight ratio

-- creates a window to display a log in reminder
-- creates a window that displays a logout reminder when it detects the action _G.Turbine.UI.Lotro.Action.Logout=268435552
-- message displays at start of logout timer

pluginName=plugin:GetName()
importPath=string.gsub(getfenv(1)._.Name,"%.Main","").."."
resourcePath=string.gsub(importPath,"%.","/").."Resources/"

locale = "en"
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de"
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr"
end
language=1
if locale=="fr" then language=2 end
if locale=="de" then language=3 end
if locale=="ru" then language=4 end -- currently not possible

if (tonumber("1,000")==1) then
	function euroNormalize(value)
		local retcode
		if value~=nil then
			retcode=tonumber((string.gsub(value,"%.",",")))
		end
		return retcode
	end
else
	function euroNormalize(value)
		local retcode
		if value~=nil then
			retcode=tonumber((string.gsub(value,",",".")))
		end
		return retcode
	end
end

string.ltrim=function(str)
	return (string.gsub(str, "^%s*(.-)", "%1"))
end
string.rtrim=function(str)
	return (string.gsub(str, "^(.-)%s*$", "%1"))
end
string.trim=function(str)
	return (string.gsub(str, "^%s*(.-)%s*$", "%1"))
end
function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback)
        else
            object[event] = {object[event], callback}
        end
    end
    return callback
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event])
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i)
                    break
                end
            end
        end
    end
end

import (importPath.."Class")
import (importPath.."VindarPatch")
-- This is the Load/Save patch which accounts for European formats, written by Vindar
-- load resource strings - this MUST occur before any windows that use strings get loaded or there will be no defaults
import (importPath.."Strings")
import (importPath.."Table")
import "Turbine"
import "Turbine.Gameplay"
import "Turbine.UI"
import "Turbine.UI.Lotro"
displayWidth=Turbine.UI.Display:GetWidth()
displayHeight=Turbine.UI.Display:GetHeight()
import (importPath.."FontSupport")
import (importPath.."DebugWindow")
import (importPath.."ColorPicker")

if _G.Turbine.UI.Lotro.Action.Logout==nil then _G.Turbine.UI.Lotro.Action.Logout=268435552 end

settings=PatchDataLoad(Turbine.DataScope.Character, "LogRemindSettings")
if settings==nil or settings.useChar==false then
	settings=PatchDataLoad(Turbine.DataScope.Account, "LogRemindSettings")
	if settings==nil then settings={} end
end
if settings.useChar==nil then
	settings.useChar=false
end
if settings.defeatShow==nil then
	settings.defeatShow=true
end
if settings.font==nil then
	settings.font=Turbine.UI.Lotro.Font.BookAntiqua36
end

if settings.color==nil then
	settings.color=0xFFFFFF
end
if settings.loginMsg==nil then
	settings.loginMsg=""
end
if settings.loginSkill~=nil then
	if tonumber(settings.loginSkill)~=nil then
		settings.loginSkill=string.format("0x%x",settings.loginSkill)
	end
	if type(settings.loginSkill)=="number" then
		settings.loginSkill=string.format("0x%x",settings.loginSkill)
	end
end
if settings.logoutMsg==nil then
	settings.logoutMsg=""
end
fontMetric=FontMetric()
import (importPath.."Options")

getColorCode=function(color)
	local R,G,B=0,1,0
	R=math.floor(color/(256*256))
	color=color-R*256*256
	G=math.floor(color/256)
	B=color-G*256
	return 1,R/256,G/256,B/256
end
setMessage=function(dstControl,msg)
-- function to determine window size based on font metric
	fontMetric:SetFont(settings.font)
	local textWidth=fontMetric:GetTextWidth(msg)
	if textWidth>displayWidth then
		textWidth=displayWidth
	end
	local textHeight=fontMetric:GetTextHeight(msg,textWidth)

	dstControl:SetSize(textWidth,textHeight)
	dstControl:SetPosition((displayWidth-dstControl:GetWidth())/2,(displayHeight-dstControl:GetHeight())/2)
	-- determine height and width of msg by settings.font
	dstControl:SetForeColor(Turbine.UI.Color(getColorCode(settings.color)))
	dstControl:SetFont(settings.font)
	dstControl:SetText(msg)
end

lrMain=Turbine.UI.Window()
lrMain:SetSize(displayWidth,displayHeight)
lrMain:SetPosition(0,0)
lrMain:SetMouseVisible(false)
lrMain:SetZOrder(501) -- the log out timer window is at zorder=500. zorder 501 will be on top of the logout timer window...
defeated=false
localPlayer=Turbine.Gameplay.LocalPlayer.GetInstance()

lrMain.msgIn=Turbine.UI.Label()
lrMain.msgIn:SetParent(lrMain)
lrMain.msgIn:SetVisible(true)
lrMain.msgIn:SetMouseVisible(false)
lrMain.msgIn:SetMultiline(true)
lrMain.msgIn:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
lrMain.skillQS=Turbine.UI.Lotro.Quickslot()
lrMain.skillQS:SetParent(lrMain)
lrMain.skillQS:SetAllowDrop(false);
lrMain.skillQS.Update=function()
	-- even though drag drop is disabled, the event fires if we drag over ourselves and it causes the quickslot to get wiped out
	if settings.loginSkill~=nil then
		local sc=Turbine.UI.Lotro.Shortcut(settings.loginSkillType,"");
		sc:SetData(settings.loginSkill);
		lrMain.skillQS:SetShortcut(sc);
	end
	lrMain.skillQS:SetWantsUpdates(false);
end
lrMain.skillQS.DragDrop=function()
	lrMain.skillQS:SetWantsUpdates(true);
end
lrMain.skillQS:SetVisible(false)
if settings.loginSkill~=nil and settings.loginSkillType~=nil then
	local sc=Turbine.UI.Lotro.Shortcut(settings.loginSkillType,"");
	sc:SetData(settings.loginSkill);
	lrMain.skillQS:SetShortcut(sc);
	lrMain.skillQS:SetVisible(true)
end

lrMain.msgOut=Turbine.UI.Label()
lrMain.msgOut:SetParent(lrMain)
lrMain.msgOut:SetVisible(false)
lrMain.msgOut:SetMouseVisible(false)
lrMain.msgOut:SetMultiline(true)
lrMain.msgOut:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)

lrMain:SetVisible(true)
Turbine.Shell.WriteLine("<rgb=#FFFFFF>Login Reminder:<rgb=#"..tostring(settings.color)..">"..tostring(settings.loginMsg).."</rgb></rgb>")
setMessage(lrMain.msgIn,settings.loginMsg)
setMessage(lrMain.msgOut,settings.logoutMsg)
lrMain.skillQS:SetPosition(displayWidth/2-16,lrMain.msgIn:GetTop()+lrMain.msgIn:GetHeight())

lrMain:SetWantsKeyEvents(true)
lrMain.KeyDown=function(sender,args)
	if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
		lrMain:SetVisible(false)
		lrMain.msgIn:SetVisible(false)
		lrMain.msgOut:SetVisible(false)
		lrMain.skillQS:SetVisible(false)
		OptionsWindow:SetVisible(false)
		OptionsWindow:SetWantsKeyEvents(false)
	end
	if (args.Action==_G.Turbine.UI.Lotro.Action.Logout) then
		lrMain:SetVisible(true)
		lrMain.msgOut:SetVisible(true)
	end
end

lrMain.RemoveShellCommands=function(sender,args)
	Turbine.Shell.RemoveCommand(lrMain.shellCmd)
end

lrMain.shellCmd=Turbine.ShellCommand()
local numberOfCommandsRegistered = Turbine.Shell.AddCommand("LogRemind",lrMain.shellCmd)

lrMain.shellCmd.GetHelp = function(sender, cmd)
	return("usage:\n  /LogRemind in|out msg\n       Sets the login or logout message\n  /LogRemind color colorcode\n       Changes text color to colorcode where colorcode is a RGB Hexcode 000000-FFFFFF\n  /LogRemind char|character\n       Sets LogRemind to character specific mode - settings will only affect this character.\n  /LogRemind acct|account\n       Sets LogRemind to account mode - default, settings will apply to any character on account not set to character mode.\n  /LogRemind skill skillcode\n       Sets LogRemind log-in skill.\n  /LogRemind status\n       Displays current LogRemind settings.\n  /LogRemind defeat on|off\n       Enables/Disables display of log-in message and skill after defeat.");
end

lrMain.shellCmd.GetShortHelp = function(sender, cmd)
	return("usage: /LogRemind [in|out msg] | [color colorcode] | [char|character] | [acct|acount] | [status] | [skill skillcode] | [defeat on|off]");
end

lrMain.shellCmd.Execute = function(sender, cmd, args)
	-- format is /logremind in|out msg
	if (string.lower(cmd) == "logremind") then
		local pos1=string.find(args," ",0,true)
		local command
		local msg=""
		if pos1~=nil and pos1>0 then
			command=string.lower(string.sub(args,1,pos1-1))
			msg=string.sub(args,pos1+1)
		else
			command=string.lower(args)
		end
		if (command=="in") then			
			settings.loginMsg=msg
			setMessage(lrMain.msgIn,msg)
			Turbine.Shell.WriteLine("Login message set to '"..msg.."'")
		elseif (command=="skill") then
			if tonumber(msg)~=nil then
				settings.loginSkillType=Turbine.UI.Lotro.ShortcutType.Skill
				settings.loginSkill=msg
				OptionsWindow.loginSkillType=settings.loginSkillType
				OptionsWindow.loginSkillData=settings.loginSkill
				Turbine.Shell.WriteLine("LogRemind Skill set to:"..msg)
			else
				Turbine.Shell.WriteLine("Invalid skill:"..msg)
			end
		elseif (command=="defeat") then
			if string.lower(msg)=="on" then
				Turbine.Shell.WriteLine("logRemind is now ON after defeat")
				settings.defeatShow=true
			elseif string.lower(msg)=="off" then
				Turbine.Shell.WriteLine("logRemind is now OFF after defeat")
				settings.defeatShow=false
			else
				Turbine.Shell.WriteLine("Invalid Option '"..msg.."', valid options are 'on' and 'off'")
			end
		elseif (command=="setup" or command=="options") then
			OptionsWindow:SetWantsKeyEvents(true)
			OptionsWindow.loginTxt:SetText(settings.loginMsg)
			OptionsWindow.loginSkillType=settings.loginSkillType
			OptionsWindow.loginSkillData=settings.loginSkill
			if settings.loginSkill~=nil and settings.loginSkillType~=nil then
				local sc=Turbine.UI.Lotro.Shortcut(settings.loginSkillType,"");
				sc:SetData(settings.loginSkill);
				OptionsWindow.loginSkill:SetShortcut(sc);
				setSkillTxt(settings.loginSkillType,settings.loginSkill)
			else
				OptionsWindow.loginSkill:SetShortcut(Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Undefined, "" ))
			end
			OptionsWindow.logoutTxt:SetText(settings.logoutMsg)
			OptionsWindow.useCharCB:SetChecked(settings.useChar)
			OptionsWindow.defeatShowCB:SetChecked(settings.defeatShow)
			OptionsWindow.newFont=settings.font
			OptionsWindow.fontLabel:SetText(Turbine.UI.Lotro.FontInfo[settings.font].name.." "..Turbine.UI.Lotro.FontInfo[settings.font].size.."pt")
			OptionsWindow.txtColor:SetColor(getColorFromCode(settings.color))
			OptionsWindow.colorLabel:SetText(settings.color)
			OptionsWindow:SetVisible(true)
		elseif (command=="status") then
			Turbine.Shell.WriteLine("<rgb=#ffff00>LogRemind Settings:</rgb>")
			Turbine.Shell.WriteLine("Log In Msg:<rgb=#80c0ff>"..tostring(settings.loginMsg).."</rgb>")
			Turbine.Shell.WriteLine("Log In Skill:<rgb=#80c0ff>"..tostring(settings.loginSkill).."</rgb>")
			Turbine.Shell.WriteLine("Log Out Msg:<rgb=#80c0ff>"..tostring(settings.logoutMsg).."</rgb>")
			Turbine.Shell.WriteLine("Char Mode:<rgb=#80c0ff>"..tostring(settings.useChar).."</rgb>")
			Turbine.Shell.WriteLine("Color:<rgb=#80c0ff>"..string.format("0x%x",settings.color).."</rgb>")
			Turbine.Shell.WriteLine("Show On Defeat:<rgb=#80c0ff>"..tostring(settings.defeatShow).."</rgb>")
		elseif (command=="out") then
			settings.logoutMsg=msg
			setMessage(lrMain.msgOut,msg)
			Turbine.Shell.WriteLine("Logout message set to '"..msg.."'")
		elseif (command=="color") then
			-- determine if msg contains a valid color code
			local color=tonumber("0x"..msg)
			if color==nil or color<0 or color>0xFFFFFF then
				Turbine.Shell.WriteLine("Invalid color code. Please use a RGB hexadecimal color code from 000000 to FFFFFF.")
			else
				settings.color=color
				Turbine.Shell.WriteLine("Log message <rgb=#"..msg..">color set to "..msg.."</rgb>")
			end
		elseif (command=="debug") then
			debugWindow:SetVisible(true)
		elseif (command=="char") or (command=="character") then
			settings.useChar=true
			Turbine.Shell.WriteLine("logRemind set to Char mode for this character")
		elseif (command=="acct") or (command=="account") then
			settings.useChar=false
			Turbine.Shell.WriteLine("logRemind set to Account mode for this character")
		elseif (command=="help") then
			Turbine.Shell.WriteLine("LogRemind usage:")
			Turbine.Shell.WriteLine("'LogRemind [in|out] message' will set the message for login or logout")
			Turbine.Shell.WriteLine("'LogRemind character' will use character specific mode (for this character)")
			Turbine.Shell.WriteLine("'LogRemind account' will use account-wide mode")
			Turbine.Shell.WriteLine("'LogRemind color colorcode' will change text color to colorcode where colorcode is a RGB Hexcode 000000-FFFFFF")
			Turbine.Shell.WriteLine("'LogRemind skill skillcode' will set the login skill quickslot")
			Turbine.Shell.WriteLine("'LogRemind defeat on|off' will enable/disable the login reminder after defeat")
			Turbine.Shell.WriteLine("'LogRemind status' will display the current logRemind settings")
			Turbine.Shell.WriteLine("ex. '/LogRemind in this is the login message' will set the login message to 'this is the login message'")
			Turbine.Shell.WriteLine("ex. '/LogRemind color 00ff00' will set the message color to green")
			Turbine.Shell.WriteLine("ex. '/LogRemind skill 0x70003F3D' will set the login skill to Find the Path")
			Turbine.Shell.WriteLine("ex. '/LogRemind skill 0x700385EE' will set the login skill to Improved Motivating Speech")
		end
	end
end
function saveSettings()
	settings.optionsLeft=OptionsWindow:GetLeft()/displayWidth
	settings.optionsTop=OptionsWindow:GetTop()/displayHeight
	PatchDataSave( Turbine.DataScope.Character, "LogRemindSettings", settings );
	if not settings.useChar then
		PatchDataSave( Turbine.DataScope.Account, "LogRemindSettings", settings );
	end
end
function UnloadPlugin()
	-- remove handler for MoraleChanged
	RemoveCallback(localPlayer,"MoraleChanged",lrMoraleChanged)
	lrMain.RemoveShellCommands()
	saveSettings()
end
lrMoraleChanged=function(sender,args)
	if settings.defeatShow then
		local morale=localPlayer:GetMorale()
		if defeated then
			if morale~=0 then
				-- has released or been rezzed, show skill
				defeated=false
				lrMain.msgOut:SetVisible(false)
				lrMain.msgIn:SetVisible(true)
				lrMain.skillQS:SetVisible(true)
				lrMain:SetVisible(true)
			end
		else
			if morale<=0 then
				defeated=true
			end
		end
	end
end
lrMain.Update=function()
	Plugins["LogRemind"].Unload = function(sender,args)
		UnloadPlugin();
	end
	AddCallback(localPlayer,"MoraleChanged",lrMoraleChanged)
	lrMain:SetWantsUpdates(false)
end
lrMain:SetWantsUpdates(true)
